#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2009  Sébastien Buchoux
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
# NMR Depaker Setup script
#
"""Setup script to build a standalone installer for Windows"""
import py2exe
from py2exe.build_exe import py2exe as build_exe
from distutils.core import setup
from glob import glob
import os
import sys
import shutil, zipfile
import matplotlib

# Useful functions
def get_files_in_dir(dirname):
    files = glob(dirname + "/*")
    
    for fname in files[:]:
        if os.path.isdir(fname):
            files.extend(get_files_in_dir(fname))
            files.remove(fname)
    
    return files

# Import version string
for line in file('lib/constants.py').readlines():
    if (line.startswith('version')):
        exec(line.strip())

# Define constants

matplot_files = matplotlib.get_py2exe_datafiles()

Py26_exclude_dlls = ["MSVCP90.dll"] # Related to Python 2.6 MS-based building crap
GTK_exclude_dlls = ["libglade-2.0-0.dll",] # Glade is not used

dist_dir = "dist/py2exe"

base_opts = {
             "dist_dir" : dist_dir,
             "excludes": ["_qtagg", "_tkagg", "tcl", "Tkinter",
                          "Tkconstants", "email"],
             "dll_excludes": Py26_exclude_dlls + GTK_exclude_dlls,
             "includes": ["cairo", "pango", "pangocairo", "atk",
                          "gobject", "gtk", "pygtk"],
             }
opts = {
        "bdist_innosetup": base_opts,
        "py2exe": base_opts,
        }

VC90_dir = "windows/Microsoft.VC90.CRT"
manifest = open("%s/Microsoft.VC90.CRT.manifest" % VC90_dir).read()
kwargs = {
          "windows": [
                      {'script': 'nmrdepaker',
                       'icon_resources': [(1, 'windows/icon.ico')],
                       'other_resources': [(24, 1, manifest)]
                       }
                      ],
          "options": opts,
          }

#===================================================================================================
# Extra setup commands
#===================================================================================================
class bdist_innosetup(py2exe.build_exe.py2exe):
    description = "create a standalone installer for Windows using py2exe and Inno Setup"
    
    def run(self):
        # Delele old mess
        if os.path.exists(dist_dir):
            shutil.rmtree(dist_dir, ignore_errors=True)
        
        # Run py2exe installer
        py2exe.build_exe.py2exe.run(self)
        
        class unzip(object):                    
            def extract(self, file, dir):
                if not dir.endswith(':') and not os.path.exists(dir):
                    os.mkdir(dir)
        
                zf = zipfile.ZipFile(file)
        
                # create directory structure to house files
                self._createstructure(file, dir)
        
                # extract files to directory structure
                for i, name in enumerate(zf.namelist()):
        
                    print "Extracting %s" % name
        
                    if not name.endswith('/'):
                        outfile = open(os.path.join(dir, name), 'wb')
                        outfile.write(zf.read(name))
                        outfile.flush()
                        outfile.close()
        
        
            def _createstructure(self, file, dir):
                self._makedirs(self._listdirs(file), dir)
        
        
            def _makedirs(self, directories, basedir):
                """ Create any directories that don't currently exist """
                for dir in directories:
                    curdir = os.path.join(basedir, dir)
                    if not os.path.exists(curdir):
                        os.makedirs(curdir)
                        print "Making dir:" + curdir
                    else:
                    	print curdir + " directory already exists!"
        
            def _listdirs(self, file):
                """ Grabs all the directories in the zip structure
                This is necessary to create the structure before trying
                to extract the file to it. """
                zf = zipfile.ZipFile(file)
        
                dirs = []
        
                for name in zf.namelist():
                    if name.endswith('/'):
                        dirs.append(name)
        
                dirs.sort()
                return dirs
        
        # Unzip GTK files to dist dir
        unzip().extract("windows/gtk.zip",
                        dist_dir)
        
        # Copy dlls and Manifest
        shutil.copy("%s/Microsoft.VC90.CRT.manifest" % VC90_dir, dist_dir)
        shutil.copy("%s/msvcm90.dll" % VC90_dir, dist_dir)
        shutil.copy("%s/msvcp90.dll" % VC90_dir, dist_dir)
        shutil.copy("%s/msvcr90.dll" % VC90_dir, dist_dir)
        
        # Update Inno Setup script
        updated_iss = "windows/setup_%s.iss" % version
        script_content = open("windows/setup_base.iss").read()
        script_content = script_content.replace("$version$", version)
        fobj = open(updated_iss, "w")
        fobj.write(script_content)
        fobj.close()
        
        # Run Inno Setup Compiler
        # Warning: Inno Setup compiler must be in PATH
        os.system("ISCC.exe " + updated_iss)
        
        # Delete updated setup script
        os.remove(updated_iss)

        
# Build data_files
globers = [("examples/DMPC/1", "*"),
           ("examples/DMPC/1/pdata/1", "*"),
          ]

data_files = []
for folder, glob_ex in globers:
    
    # Retrieve files in folder
    files = glob(os.path.join(folder, glob_ex))
    
    # Remove folders
    for fname in files[:]:
        if not os.path.isfile(fname):
            files.remove(fname)
    
    data_files.append((folder, files))

matplotlib_files = []
for package, files in matplotlib.get_py2exe_datafiles():
    for fname in files[:]:
        if not os.path.isfile(fname):
            files.remove(fname)
    if len(files) > 0:
        matplotlib_files.append((package, files))

# Since py2exe ignores package_data, we need to add data files explicitly
data_files.extend(matplotlib_files)

# Retrieve files in folder
files = glob("lib/data/images/*.png")

# Remove folders
for fname in files[:]:
    if not os.path.isfile(fname):
        files.remove(fname)

data_files.append(("nmrdepaker-data/images", files))            

#===================================================================================================
# Run Setup
#===================================================================================================
# Hack to be sure to execute the proper builder
sys.argv = sys.argv[0:] + ["bdist_innosetup"]

setup(name='nmrdepaker',
      version=version,
      description='Program to perform NMR spectra "De-Pake-ing"',
      author='Sébastien Buchoux',
      author_email='sebastien@nmrfriend.org',
      url='https://launchpad.net/nmrdepaker',
      packages=['nmrdepaker', 'nmrdepaker.mpl'],
      package_dir={'nmrdepaker': 'lib',
                   'nmrdepaker.mpl' : 'lib/mpl'},
      package_data={'nmrdepaker': ['data/images/*.png']},
      data_files=data_files,
      scripts=["nmrdepaker"],
      cmdclass={"bdist_innosetup" : bdist_innosetup},
      **kwargs
      )
