#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2009  Sébastien Buchoux
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
# 
#
import gobject
import gtk

from xml.etree import ElementTree as ET
from warnings import warn

from bases import ApplicationComponent, BaseApplication, MenuItemCommand
from util import action_search

#===============================================================================
# Application components
#===============================================================================

# Menu
class MenuComponent(ApplicationComponent):
    """Menu class"""
    def __init__(self, application=None):
        # Call ancestor __init__
        ApplicationComponent.__init__(self,
                                      name="menu",
                                      widget=None,
                                      application=application)
        
        # UI
        self.ui_manager = gtk.UIManager()
        self.ui = None
        self._merge_id = None
        
        # Action groups
        self.action_groups = {}
    
    def _default_ui(self):
        """Return default UI as an ElementTree Element"""
        xml_ui = ET.Element("ui")
        ET.SubElement(xml_ui, "menubar", {"name" : "MenuBar"})
        return xml_ui
    
    def add_ui_to_manager(self, xml_ui, top_sep=False, bottom_sep=False):
        """Add a string UI to UIManager"""
        if self.ui is None:
            self.ui = xml_ui
        else:
            item = None
            parent = xml_ui
            tmp_parent = xml_ui
            
            while len(tmp_parent.getchildren()) != 0:
                parent = tmp_parent
                item = parent[0]
                
                tmp_parent = tmp_parent[0]
            
            # Check if parent is already registered
            ui_parent = self.ui[0] # menubar
            parent_action = parent.get("action")
            if parent.tag != "menubar":
                ui_parent = action_search(ui_parent, parent_action)
            
            if ui_parent is None:
                raise ValueError("Unregistered item parent: %s" % repr(parent))

            action = item.get("action")
            position = item.get("pos")
            
            ui_item = None
            for child in ui_parent.getchildren():
                if child.get("action") == action:
                    ui_item = child
                    break
            
            if ui_item is not None:
                warn("Action '%s' already defined, nothing to do!" % action)
            else:
                elems = dict()
                names = list()
                
                for elem in ui_parent.getchildren():
                    if elem.tag == "separator":
                        elem_name = elem.get("parent")
                        elem_key = "%s_sep" % elem.get("type")
                    else:
                        elem_name = elem.get("action")
                        elem_key = "item"
                    
                    if elem_name not in names:
                        if elem_name is None:
                            raise ValueError("Element name not set!")
                            
                        names.append(elem_name)
                        elems[elem_name] = dict()
                    
                    elems[elem_name][elem_key] = elem
                    
#                if item.tag == "menu":
#                    print "\n####"
#                    print names
#                    print elems
                insert_pos = 0
                
                
                if len(ui_parent) != 0:
                    if position == "top":
                        for i, name in enumerate(names):
                            elem = elems[name]["item"]
                            if elem.get("pos") != "top":
                                insert_pos = i
                                break
                    else:
                        for i in range(len(names), 0 , -1):
                            elem = elems[names[i-1]]["item"]
                            if elem.get("pos") != "bottom":
                                insert_pos = i
                                break
                
#                if item.tag == "menu":
#                    print "%s wants to be inserted at position %i" % (action, insert_pos)
                
                # Insert item
                names.insert(insert_pos, action)
                elems[action] = {"item": item}
                
                #ui_parent.insert(insert_pos, item) # insert elem
                
                
                if top_sep:
                    attrib = dict()
                    attrib["parent"] = action
                    attrib["type"] = "top"
                    sep = ET.Element("separator", attrib)
                    #ui_parent.insert(insert_pos - 1, sep)
                    elems[action]["top_sep"] = sep
                
                if bottom_sep:
                    attrib = dict()
                    attrib["parent"] = action
                    attrib["type"] = "bottom"
                    sep = ET.Element("separator", attrib)
                    #ui_parent.insert(insert_pos + 1, sep)
                    elems[action]["bottom_sep"] = sep
            
            # Update parent element
            parent_attrib = dict(ui_parent.attrib)
            ui_parent.clear()
            ui_parent.attrib = parent_attrib
            
#            if len(ui_parent) != 0:
#                print "FUCK!!!!!"
#                print ET.tostring(ui_parent)
#            
#            if item.tag == "menu":
#                print elems
#                print "####"
#                print "Building parent"
#            print elems
#            print "####"
#            print "Building parent"
            
            pos = 0
            for name in names:
                if "top_sep" in elems[name]:
                    ui_parent.insert(pos, elems[name]["top_sep"])
                    pos += 1
#                    if item.tag == "menu":
#                        print ET.tostring(elems[name]["top_sep"])
#                    print ET.tostring(elems[name]["top_sep"])
                
                ui_parent.insert(pos, elems[name]["item"])
                pos += 1
#                if item.tag == "menu":
#                    print ET.tostring(elems[name]["item"])
#                print ET.tostring(elems[name]["item"])
                
                if "bottom_sep" in elems[name]:
                    ui_parent.insert(pos, elems[name]["bottom_sep"])
                    pos += 1
#                    if item.tag == "menu":
#                        print ET.tostring(elems[name]["bottom_sep"])
#                    print ET.tostring(elems[name]["bottom_sep"])
                    
#            if item.tag == "menu":
#                print "####\nRebuilt UI:"
#                print ET.tostring(ui_parent)
#                print "####"
#            print "####\nRebuilt UI:"
#            print ET.tostring(ui_parent)
#            print "####"  
        
                
        if self._merge_id is not None:
            self.ui_manager.remove_ui(self._merge_id)
            self.ui_manager.ensure_update()
        
        # Update ui_manager
        string_ui = ET.tostring(self.ui)
#        print "UI passed to UI manager"
#        print string_ui
        self._merge_id = self.ui_manager.add_ui_from_string(string_ui)
        self.ui_manager.ensure_update()
        
        # Update widget
        self.widget = self.ui_manager.get_widget("/MenuBar")
        
    def setup(self):
        """Setup Menu"""
        
        # Add default UI
        self.add_ui_to_manager(self._default_ui())
        
        # Retrieve widget
        self.widget = self.ui_manager.get_widget("/MenuBar")
    
    def _build_ui(self, menu_path, action, ui_type, position):
        """Build UI"""
        
        xml_ui = self._default_ui()
        xml_menubar = xml_ui.find("menubar")
        
        xml_parent = xml_menubar
        for menu in menu_path.split("/"):
            if menu != "":
                xml_menu = ET.SubElement(xml_parent, "menu", {"action" : menu})
                xml_parent = xml_menu
        
        attrib = {"action" : action}
        if position is not None:
            attrib["pos"] = position
        
        ET.SubElement(xml_parent, ui_type, attrib)
        
        return xml_ui
    
    def add_action(self, menu_path, action_entry, group, ui_type, pos, seps):
        """Add to UI"""
        
        # Retrieve info from action entry
        action = action_entry[0]        
        try:
            callback = action_entry[5]
        except IndexError:
            callback = None
        
        # Build UI to be merged
        xml_ui = self._build_ui(menu_path, action, ui_type, pos)
        
        
        # Handle action group
        try:
            action_group = self.action_groups[group]
        except KeyError:
            #print "Creating ActionGroup: %s" % group
            action_group = gtk.ActionGroup(group)
            self.action_groups[group] = action_group
            self.ui_manager.insert_action_group(action_group, -1)
        
        # Handle callback
        if callback is not None:
            action_entry = list(action_entry)
            action_entry[5] = self._action_cb
            user_data = callback
        else:
            user_data = None
        
        for gtkaction in action_group.list_actions():
            if action == gtkaction.get_name():
                label = action_entry[2]
                gtklabel = gtkaction.get_property("label")
                
                if label != gtklabel:
                    warn("%s action already defined with label: %s" % (action,
                                                                      gtklabel))
                return
        
        # Add action to action group
        action_entry = tuple(action_entry)
        action_group.add_actions((action_entry,), user_data=user_data)

        # Finally update UI
        self.add_ui_to_manager(xml_ui, seps[0], seps[1])
        
    def add_menu(self, menu_path, action_entry, group="default",
                 position=None, separators=(False, False) ):
        """Add Menu"""
        self.add_action(menu_path, action_entry, group, "menu", position,
                        separators)
        
    def add_item(self, menu_path, action_entry, group="default",
                 position=None, separators=(False, False)):
        """Add Menu item"""
        self.add_action(menu_path, action_entry, group, "menuitem", position,
                        separators)
    
    def set_group_sensitive(self, group_name, sensitive=None):
        """
        Set *group_name*-named group's sensitivity.
        
        If *sensitivity* is not given, toggle group's sensitivity
        """
        try:
            group = self.action_groups[group_name]
        except KeyError:
            return
        
        if sensitive is None:
            sensitive = not group.get_sensitive()
        group.set_sensitive(sensitive)
        
    def _action_cb(self, action, command_cb):
        #print "Running action callback (Command: %s)" % repr(command_cb)
        command_cb()


# Status bar       
class StatusbarComponent(ApplicationComponent):
    """Status bar class"""
    def __init__(self, application=None):
        # Call ancestor __init__
        ApplicationComponent.__init__(self,
                                      name="statusbar",
                                      widget=gtk.Statusbar(),
                                      application=application)
        
        self.context_id = -1
    
    def setup(self):
        """Setup Status bar"""
        self.widget.set_has_resize_grip(True)
    
    def pop(self):
        self.widget.pop(self.context_id)
    
    def push(self, text):
        self.widget.push(self.context_id, text)

# About dialog
class AboutComponent(ApplicationComponent):
    """Class to handle display of information about an application"""
    def __init__(self, application=None):
        # Call ancestor __init__
        ApplicationComponent.__init__(self,
                                      name="about",
                                      widget=None,
                                      application=application)
        
    
    def setup(self):
        """About setup"""
        # Add command
        self.add_command(ShowAboutCommand(), setup=False)
        
        # Set widget up
        self.widget = gtk.AboutDialog()
        self.widget.set_name(self.application.name)
        self.widget.set_version(self.application.version)
   
#===============================================================================
# Application Related commands
#===============================================================================
class ShowAboutCommand(MenuItemCommand):
    """General Exit command"""
    def __init__(self):
        
        # Item
        item_entry = ("about",
                      gtk.STOCK_ABOUT,
                      "A_bout...",
                      None,
                      "About this program",
                      self.run)
        item_group = "default"
        item_position = "bottom"
        
        # Menu
        menu_entry = ("Help", None, "_Help")
        menu_group = "default"
        menu_position = "bottom"
        
        MenuItemCommand.__init__(self,
                                 "about_command",
                                 item_entry,
                                 menu_entry,
                                 item_group=item_group,
                                 item_position=item_position,
                                 menu_group=menu_group,
                                 menu_position=menu_position,
                                 )
        
    def run(self):
        """Show about component's widget"""
        about = self._commander.get_component("about")
        
        # Run GTK Dialog
        about.widget.run()
        
        # Hide dialog
        about.widget.hide()

#===============================================================================
# Application related events    
#===============================================================================
#class ExitEvent(Event):
#    """Event triggered on exit"""
#    name = "exit_event"

#===============================================================================
# Application class
#===============================================================================
class Application(BaseApplication):
    """
    Application base class.
    
    TODO: complete docstring
    """
    def __init__(self, name="Generic application", version="1.0"):
        # Call ancestor __init__
        BaseApplication.__init__(self, name, version)

        # Update special components
        self.menu = MenuComponent(application=self)
        self.main_display = None
        self.status_bar = StatusbarComponent(application=self)
        
        # Gui related
        self.window = gtk.Window()
    
    def _setup_window(self):
        """Window setup"""
        
        # Window setup
        self.window.vbox = gtk.VBox()
        self.window.add(self.window.vbox)
        
        self.window.set_title(self.name)
        self.window.set_default_size(800, 600)
        self.window.set_position(gtk.WIN_POS_CENTER_ALWAYS)
        self.window.set_resizable(True)
        
        # Pack widgets
        # menu
        if self.menu is not None:
            self.window.vbox.pack_start(self.menu.widget, expand = False)
        
        # main display
        if self.main_display is not None:
            self.window.vbox.pack_start(self.main_display.widget, expand = True)
        
        # status bar
        if self.status_bar is not None:
            self.window.vbox.pack_end(self.status_bar.widget, expand = False)
        
        # Connect accelerators
        self.window.add_accel_group(self.menu.ui_manager.get_accel_group())
    
#    def _setup_commands(self):
#        """Setup application commands"""
#        BaseApplication._setup_commands(self)
#    
#    def _setup_events(self):
#        """Setup events"""
#        BaseApplication._setup_events(self)
          
    def setup(self, forced=False):
        """Application setup."""
        if not forced:
            if self._is_setup:
                return
        
        # Connect proxy callbake BEFORE adding any actions
        self.menu.ui_manager.connect('connect-proxy',
                                     self._connect_ui_proxy_cb)
        
        self.menu.ui_manager.connect('disconnect-proxy',
                                     self._disconnect_ui_proxy_cb)
        
        # Call ancestor
        BaseApplication.setup(self, forced)
        
        # Handle callbacks
        self.window.connect("delete_event", self.gtk_callbacker, "exit_command")
        
        self._is_setup = True
        
    def gtk_callbacker(self, *args):
        command = args[-1]
        return self.run_command(command)
        
    def run(self):
        BaseApplication.run(self)
        
        # Show GUI
        self.window.show_all()
        
        # Enable thread
        #gtk.gdk.threads_init()
        
        # Get into GTK main loop
        gtk.main()

    def exit(self):
        BaseApplication.exit(self)
        
        # Destroy window
        self.window.destroy()
        
        # Exit GTK main loop
        gtk.main_quit()
        
    def _menu_item_select_cb(self, menuitem, tooltip):
        self.status_bar.push(tooltip)
    
    def _menu_item_deselect_cb(self, menuitem):
        self.status_bar.pop()
        
    def _connect_ui_proxy_cb(self, uimgr, action, widget):
        """see: http://faq.pygtk.org/index.py?req=show&file=faq11.010.htp
        and: http://www.mail-archive.com/pygtk@daa.com.au/msg12949.html"""
        #print 'connect-proxy widget:', widget
        tooltip = action.get_property('tooltip')
        if isinstance(widget, gtk.MenuItem) and tooltip:
            cid = widget.connect(
                'select', self._menu_item_select_cb, tooltip)
            cid2 = widget.connect(
                'deselect', self._menu_item_deselect_cb)
            widget.set_data('app::connect-ids', (cid, cid2))

    def _disconnect_ui_proxy_cb(self, uimgr, action, widget):
        #print 'disconnect-proxy widget:', widget
        cids = widget.get_data('app::connect-ids') or ()
        for cid in cids:
            widget.disconnect(cid)
        
if __name__ == "__main__":
    raw_input("Launching a generic application... [Press any key]")
    
    app = Application()
    app.add_component(AboutComponent())
    app.run()
    
