#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2009  Sébastien Buchoux
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
# 
#
import numpy
from numpy import fft as np_fft

def fft_depake(hz_axis, ppm_axis, spc, aq_time):
    
    # Rebuild FID
    tmp_fid = np_fft.ifft(np_fft.ifftshift(spc))
    
    td = len(tmp_fid)
    
    # Rebuild time axis
    time_axis = numpy.linspace(0, aq_time, td)
    
    # Perform FFT depaking
    tmp_fid *= numpy.sqrt(time_axis)
    
    # built halves
    right = np_fft.fftshift(np_fft.fft(tmp_fid)) * (1+ 1j)
    left = np_fft.fftshift(np_fft.fft(tmp_fid)) * (1- 1j)
    
    # built depaked spectrum
    new_spc = numpy.concatenate([left[:td/2], right[td/2:]])
        
    new_hz_axis = hz_axis * 2
    new_ppm_axis = ppm_axis * 2
    
    return (new_hz_axis, new_ppm_axis, new_spc)