#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2009  Sébastien Buchoux
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
# 
#
from __future__ import division

# global imports
import math
import numpy
import random

def _calc_m1_zero(spc, left, right):
    """Calculate first moment zero."""
        
    ar1 = 0.0
    ar2 = 0.0
    for i in range(left, right):
        ar1 += spc[i]
        ar2 += spc[i]*i
    
    return int(round(ar2/ar1))
    
def calculate_moments(hz_axis, spc, left, right, pivot):
    # Perform baseline correction
    noise = 0.0
    n_points = 0
    
    for val in spc[:left]:
        noise += val
        n_points += 1
    
    for val in spc[right:]:
        noise += val
        n_points += 1
    
    noise /= n_points
    
    # Substract noise to spectrum to correct baseline
    spc = spc - noise
    
    
    m1_zero = _calc_m1_zero(spc, left, right)
    
    two_pi = math.pi * 2
    
    m0 = m1 = m1_pivot = m1sym = m1sym_pivot = m2 = m2_pivot = 0.0

    for i in range(left, right):
        delta_pivot = hz_axis[i] - hz_axis[pivot]
        delta_m1_zero = hz_axis[i] - hz_axis[m1_zero]
        
        spc_val = spc[i]
        
        # m0
        m0 += spc_val
        
        # m1 & m1sym
        m1_pivot += spc_val * delta_pivot
        m1sym_pivot += spc_val * abs(delta_pivot)
        
        m1 += spc_val * delta_m1_zero
        m1sym += spc_val * abs(delta_m1_zero)
        
        # m2 
        m2_pivot += spc_val * delta_pivot ** 2
        m2 += spc_val * delta_m1_zero ** 2
        
    m1_pivot /= m0
    m1sym_pivot /= m0
    m2_pivot /= m0
    
    m1 /= m0
    m1sym /= m0
    m2 /= m0
        
    
    ret_val = {"left_limit": hz_axis[left],
               "right_limit": hz_axis[right],
               "pivot": hz_axis[pivot],
               "M1_zero": hz_axis[m1_zero],
               "M1_with_zero": m1 * two_pi,
               "M1_with_pivot": m1_pivot * two_pi,
               "M1sym_with_zero": m1sym * two_pi,
               "M1sym_with_pivot": m1sym_pivot * two_pi,
               "M2_with_zero": m2 * two_pi ** 2,
               "M2_with_pivot": m2_pivot * two_pi ** 2,
               }
    
    return ret_val
        