#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2009  Sébastien Buchoux
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
# 
#
import numpy

# local imports
import documentmanager

class NMRSpectrum(object):
    def __init__(self, spectrum, ppm_axis, hz_axis, aq_time, title="Unknown spectrum", name="Unknown"):
        self.spectrum = spectrum
        self.ppm_axis = ppm_axis
        self.hz_axis = hz_axis
        self.title = title
        self.aq_time = aq_time
        self.name = name
    
    def get_time_axis(self):
        return numpy.linspace(0, self.aq_time, len(self.spectrum))
    
    def get_spectrum(self):
        return self.spectrum.real
    
    y = property(get_spectrum)
    
    def get_hz_axis(self):
        return self.hz_axis
    
    x = property(get_hz_axis)
    

class NMRSpectrumDocument(documentmanager.Document):
    def __init__(self, nmr_spectrum, name="unknown spectrum"):
        documentmanager.Document.__init__(self, name, "NMR spectrum")
        
        self.nmr_spectrum = nmr_spectrum
    
    def get_spectrum(self):
        return self.nmr_spectrum