#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2009 Sébastien Buchoux
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
# Miscellaneous utility functions
#
"""Miscellaneous utility functions"""

# global imports
try:
    from numbers import Number # New in python 2.6
except ImportError:
    Number = [int, float]

import numpy

def isnumber(obj):
    """
    Check if python object *obj* is a number.
    Return True if yes, False if not.
    """
    return isinstance(obj, Number)

def str_to_bool(text):
    """Converts given string to boolean.
    Returns equivalent boolean or string if no equivalent found.
    """
    text = text.strip().lower()
  
    if text in ("true", "1", "on"):
        return True
    elif text in ("false", "0", "off"):
        return False
    else:
        raise ValueError("Not a boolean: %s" % text)

def str_to_number(text):
    """
    Converts a string to the most adequate number format if possible.
    Return a int/float or raise a ValueError if not possible
    """
    text = text.strip()
    try:
        return int(text)
    except ValueError:
        try:
            return float(text)
        except ValueError:
            raise ValueError ("Not a valid number: %s" % text)

def action_search(xml_tree, action):
    """
    Recursively search an action in a xml tree.
    
    TODO: complete docstring.
    """
    
    for child in xml_tree.getchildren():
        found = action_search(child, action)
        if found is not None:
            return found
    
    if xml_tree.get("action") == action:
        return xml_tree
    else:
        return None

def closest_index(np_array, value):
    temp = numpy.abs(np_array - value)
    
    return temp.argmin()
