#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2009  Sébastien Buchoux
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
# NMR Depaker Setup script
#
"""NMR Depaker Setup script"""

from distutils import log
from distutils.core import setup
from distutils.command.install import install as _install
from distutils.command.sdist import sdist as _sdist
from glob import glob
import os
import sys

# Useful functions
def get_files_in_dir(dirname):
    files = glob(dirname + "/*")
    
    for fname in files[:]:
        if os.path.isdir(fname):
            files.extend(get_files_in_dir(fname))
            files.remove(fname)
    
    return files

# Import version string
for line in file('lib/constants.py').readlines():
    if (line.startswith('version')):
        exec(line.strip())

INSTALLED_FILES = '.nmrdepaker_files'

# Set folder for data files
#if sys.platform == "linux2":
#    DATA_PREFIX = "share/nmrdepaker/data"
#elif sys.platform == "win32":
#    DATA_PREFIX = os.path.expanduser("~/Application Data/nmrdepaker/data")
#else:
#    DATA_PREFIX = os.path.expanduser("~/.nmrdepaker/data")
#DATA_PREFIX = os.path.realpath(DATA_PREFIX)

#===================================================================================================
# Extra setup commands
#===================================================================================================

# the two following classes are modified but were stolen from fusion-icon setup.py
# and were originally stolen from ccsm
class install_logged (_install):
    description = "install everything using a log file to enable uninstallation."
    
    def run (self):

        _install.run(self)
        outputs = self.get_outputs()
        data = '\n'.join(outputs)
        try:
            f = open(INSTALLED_FILES, 'w')
        except:
            self.warn ('Could not write installed files list %s' % INSTALLED_FILES)
            return

        f.write(data)
        f.close()
        
        log.info("Installed files list succesfully saved to %s" % INSTALLED_FILES)

class uninstall_logged(_install):
    description = "uninstall everything using a log file"
    
    def run(self):
        try:
            files = file(INSTALLED_FILES, 'r').readlines()
        except:
            self.warn('Could not read installed files list %s' %INSTALLED_FILES)
            return

        for f in files:
            print 'Uninstalling %s' %f.strip()
            try:
                os.unlink(f.strip())
            except:
                self.warn('Could not remove file %s' %f)
        os.remove(INSTALLED_FILES)

# Build data_files
globers = [("examples/DMPC/1", "*"),
           ("examples/DMPC/1/pdata/1", "*"),
          ]

data_files = []
for folder, glob_ex in globers:
    
    # Retrieve files in folder
    files = glob(os.path.join(folder, glob_ex))
    
    # Remove folders
    for fname in files[:]:
        if not os.path.isfile(fname):
            files.remove(fname)
    
    data_files.append((os.path.join("share/nmrdepaker", folder), files))
    #data_files.append((folder, files))     

#===================================================================================================
# Run Setup
#===================================================================================================
cmdclass={
           'uninstall_logged': uninstall_logged,
            'install_logged': install_logged,
         }

setup(name='nmrdepaker',
      version=version,
      description='Program to perform NMR spectra "De-Pake-ing"',
      author='Sébastien Buchoux',
      author_email='sebastien@nmrfriend.org',
      url='https://launchpad.net/nmrdepaker',
      packages=['nmrdepaker', 'nmrdepaker.mpl'],
      package_dir={'nmrdepaker': 'lib',
                   'nmrdepaker.mpl' : 'lib/mpl'},
      package_data={'nmrdepaker': ['data/images/*.png']},
      data_files=data_files,
      scripts=["nmrdepaker"],
      cmdclass=cmdclass,
      )
